<?php

///
//----------------------------Integration------------------------------//
///

// Register the integration's name, icons, and code 
add_action( 'uncanny_automator_add_integration', 'bravepop_add_integration_func', 10 ,1 ); // Function names need to be unique

/**
 * Add integration data to Automator
 */
function bravepop_add_integration_func() {

   global $uncanny_automator;

   $uncanny_automator->register_integration(
      'BRAVE', // Integration Code
      array(
         'name'        => 'Brave Conversion Engine', // Integration Name
         'icon_16'     => BRAVEPOP_PLUGIN_PATH . 'assets/images/uncanny/integration-brave-icon-16.png',
         'icon_32'     => BRAVEPOP_PLUGIN_PATH . 'assets/images/uncanny/integration-brave-icon-32.png',
         'icon_64'     => BRAVEPOP_PLUGIN_PATH . 'assets/images/uncanny/integration-brave-icon-64.png',
         'logo'        => BRAVEPOP_PLUGIN_PATH . 'assets/images/uncanny/integration-brave-logo.png',
         'logo_retina' => BRAVEPOP_PLUGIN_PATH . 'assets/images/uncanny/integration-brave-logo_retina.png',
      )
   );
}



/**
 * Check if the plugin we are adding an integration for is active based on integration code
 *
 * @param bool   $status If the integration is already active or not
 * @param string $code   The integration code
 *
 * @return bool $status
 */
add_filter( 'uncanny_automator_maybe_add_integration', 'bravepop_maybe_add_integration', 11, 2 );
function bravepop_maybe_add_integration( $status, $code ) {

   if ( 'BRAVE' === $code ) {
      if ( defined('BRAVEPOP_PLUGIN_PATH') ) {
         $status = true;
      } else {
         $status = false;
      }
   }

   return $status;
}



///
//---------------------------Action-----------------------------//
///

// Add action during the "uncanny_automator_add_integration_triggers_actions_tokens" do_action()
add_action( 'uncanny_automator_add_integration_triggers_actions_tokens', 'uncanny_automator_actions_send_email' );
function uncanny_automator_actions_send_email() {

   global $uncanny_automator;

   $action = array(
		'author'             => 'Brave',
		'support_link'       => 'https://getbrave.io/contact',
		'integration'        => 'BRAVE',
		'code'               => 'OPENBRAVEPOPUP',
		'sentence'           => sprintf( __( 'Open a {{Brave Popup:%s}}', 'bravepop' ), 'BRAVEPOPUPID' ),
		'select_option_name' => __( 'Open {{Popup}}', 'bravepop' ),
		'priority'           => 5,
		'accepted_args'      => 1,
		'execution_function' => 'bravepop_uncanny_open_popup',
		'options'      => [
			bravepop_uncanny_all_brave_popups(null, 'BRAVEPOPUPID'),
		],
	);

   $uncanny_automator->register_action( $action );
}

function bravepop_uncanny_all_brave_popups( $label = null, $option_code = 'BRAVEPOPUPID' ) {
   global $uncanny_automator;
   if ( ! $label ) {
      $label = esc_attr__( 'Popup', 'bravepop' );
   }

   //Load All Popups
   $args = array( 'numberposts' => -1,  'post_type' => 'popup', 'meta_query' => array( array( 'key' => 'popup_type', 'value' => 'popup','compare' => 'LIKE' ) )  );
   $options = $uncanny_automator->helpers->recipe->options->wp_query( $args, false, __( 'Any Popup', 'bravepop' ) );


   $option = [
      'option_code'     => $option_code,
      'label'           => $label,
      'input_type'      => 'select',
      'required'        => true,
      'options'         => $options,
      'relevant_tokens' => [
         $option_code => esc_attr__( 'Popup title', 'bravepop' ),
      ]
   ];

   return $option;
}

/**
 * The function that performs the action
 *
 * @param int   $user_id     The user ID that is completing the action
 * @param array $action_data The action's option data
 * @param int   $recipe_id   ID of the recipe associated with the action
 */
function bravepop_uncanny_open_popup( $user_id, $action_data, $recipe_id ) {

   global $uncanny_automator;
   if ( isset( $action_data['meta'][ 'BRAVEPOPUPID' ] ) ) {

      $popup_id = $action_data['meta'][ 'BRAVEPOPUPID' ];

      $popup_id = absint( $popup_id );

      add_action( 'wp_head', function () use ( $popup_id ) {
         global $bravepop_global;

         $alreadyLoaded = false;

         foreach ($bravepop_global['current_popups'] as $key => $item) {
            if($alreadyLoaded === false && ($item->id === $popup_id && $item->status === 'published')){
               $alreadyLoaded = true;
            }
         }

         if($alreadyLoaded === false){
            $customContent = json_decode(get_post_meta($popup_id, 'popup_data', true));
            $customContent->settings->trigger->triggerType = 'load';
            new BravePop_Popup($popup_id, 'popup', false, false, json_encode($customContent) ); //insert the popup to the current page
         }

         ?>
         <?php
      }, 9 );

      $uncanny_automator->complete_action( $user_id, $action_data, $recipe_id );
   }

   return;
}


?>